#ifndef ICONHADLER_H
#define ICONHADLER_H

#include <QtGui/QImage>
#include <QtCore/QFile>



// These next two structs represent how the icon information is stored
// in an ICO file.
typedef struct
{
    quint8	bWidth;               // Width of the image
    quint8	bHeight;              // Height of the image (times 2)
    quint8	bColorCount;          // Number of colors in image (0 if >=8bpp) [ not ture ]
    quint8	bReserved;            // Reserved
    quint16	wPlanes;              // Color Planes
    quint16	wBitCount;            // Bits per pixel
    quint32	dwBytesInRes;         // how many bytes in this resource?
    quint32	dwImageOffset;        // where in the file is this image
} ICONDIRENTRY, *LPICONDIRENTRY;
#define ICONDIRENTRY_SIZE 16

typedef struct
{
    quint16	idReserved;   // Reserved
    quint16	idType;       // resource type (1 for icons)
    quint16	idCount;      // how many images?
    ICONDIRENTRY	idEntries[1]; // the entries for each image
} ICONDIR, *LPICONDIR;
#define ICONDIR_SIZE    6       // Exclude the idEntries field

typedef struct {				// BMP information header
    quint32  biSize;				// size of this struct
    quint32  biWidth;				// pixmap width
    quint32  biHeight;				// pixmap height
    quint16  biPlanes;				// should be 1
    quint16  biBitCount;			// number of bits per pixel
    quint32  biCompression;			// compression method
    quint32  biSizeImage;				// size of image
    quint32  biXPelsPerMeter;			// horizontal resolution
    quint32  biYPelsPerMeter;		// vertical resolution
    quint32  biClrUsed;				// number of colors used
    quint32  biClrImportant;			// number of important colors
} BMP_INFOHDR ,*LPBMP_INFOHDR;
#define BMP_INFOHDR_SIZE 40


bool writeIconDirEntry(QIODevice *iodev, const ICONDIRENTRY &iconEntry);
bool writeIconDir(QIODevice *iodev, const ICONDIR &iconDir);
bool writeBMPInfoHeader(QIODevice *iodev, const BMP_INFOHDR &header);
bool writeIco(QIODevice * device, const QList<QImage> & images);

#endif // ICONHADLER_H
