#include <QtCore/QCoreApplication>
#include <stdio.h>
#include <QTextStream>
#include <QTextCodec>
#include <QDir>
#include <QFileInfo>
#include <QProcess>
#include <QRegExp>
#include <QList>
#include <QDebug>
#include <QImage>
#include "iconhadler.h"


QStringList getAvailSizes(const QString & rootDir)
{
  QStringList result;
  QDir pwd(rootDir);
  QFileInfoList list = pwd.entryInfoList(QDir::Dirs);
  QRegExp rx("\\d+x\\d+");
  foreach(QFileInfo fi, list)
  {
    if(rx.indexIn(fi.fileName()) >= 0)
    {
      result.append(rx.cap(0));
    }
  }
  return result;
}

QList<QImage> getImages(const QString & rootDir, const QStringList & sizes, const QString & pngFileName)
{
  QList<QImage> result;
  foreach(QString sizeFolder, sizes)
  {
    QString fileName = rootDir + "/" + sizeFolder + "/" + pngFileName;
    if(QFile().exists(fileName))
    {
      QImage img(fileName);
      result.append(img);
    }

  }
  return result;
}

QStringList findIcons(const QString & rootDir)
{
  QStringList files;
  QDir root(rootDir);
  QFileInfoList dirsSize = root.entryInfoList(QDir::Dirs);
  QRegExp rx("\\d+x\\d+");
  foreach(QFileInfo fi, dirsSize)
  {
    if(rx.indexIn(fi.fileName()) >= 0)
    {
      QDir ds(fi.absoluteFilePath());
      QFileInfoList folders = ds.entryInfoList(QDir::Dirs);
      foreach(QFileInfo iFolder, folders)
      {
        QDir folder(iFolder.absoluteFilePath());
        QStringList pngs = folder.entryList(QStringList() << "*.png");
        foreach(QString filePng, pngs)
        {
          QString file = QString("%1/%2").arg(iFolder.fileName(), filePng);
          if(!files.contains(file)) files.append(file);
        }

      }
    }
  }

  return files;
}


void WriteIcons(const QString & rootDir)
{
  QDir root(rootDir);


  QStringList sizes = getAvailSizes(rootDir);
  QStringList files = findIcons(rootDir);
  if(files.size() > 0)
  {
    root.mkdir("ico");
    root.cd("ico");

    qDebug() << "Found sizes: " << sizes.join(", ");
    qDebug() << "Found icons: " << files.join("\n");

    foreach(QString png, files)
    {
      QString folder = png.split("/").first();
      if(!root.exists(folder))
        root.mkdir(folder);
      QFileInfo fi(png.split("/").last());

      QString icoName = rootDir + "/ico/" + folder + "/" + fi.baseName()+".ico";
      QList<QImage> images = getImages(rootDir, sizes, png);
      if(images.size() > 0)
      {
        qDebug() << "Writting: " << icoName;
        QFile ico(icoName);
        if(ico.open(QIODevice::ReadWrite | QIODevice::Truncate))
        {
         writeIco(&ico, images);
        }
        else qDebug() << "Unable create file: " << icoName;
        ico.close();
      }
      else
        qDebug() << "No images for: " << fi.baseName();
    }
  }
  else
    qDebug() << "No images found to convert.";
}


int main(int argc, char *argv[])
{
  QCoreApplication a(argc, argv);
  QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
  QString rootDir = QDir::toNativeSeparators(QDir::current().absolutePath());
  QTextStream ts(stdout);
  ts << "icofrompng-2009.08"<<endl<<"Copyright (c) 2009 Victor V. Zinkevich <vicking@yandex.ru>"<< endl <<"http://vicking.narod.ru" << endl;
  ts << "License: GPL v.3" << endl << endl;
  if(a.arguments().count() > 1)
  {
    QString arg = a.arguments().last();
    if(arg == "--help")
    {
      ts << "USAGE: icofrompng [<iconset_directory>|--help]" << endl << endl;
      ts << "  <iconset_directory>    Specify a root directory of iconset." << endl <<
            "                         It must contain directory such as '16x16/actions'."
 << endl << "                         If not specified the current directory will be used."
            << endl << endl;
      ts << "  --help                 Prints this screen." << endl;
      return 0;
    }
    rootDir = arg;
  }
  qDebug() << "Current directory: " << QDir::toNativeSeparators(QDir::current().absolutePath());
  qDebug() << "'Root' directory: " << rootDir;

  WriteIcons(QDir::fromNativeSeparators(rootDir));

  return 0;
}
